/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;
import com.nonesole.web.platform.sys.PlatformXMLParser;
import com.nonesole.web.platform.sys.function.Function;
import com.nonesole.web.platform.sys.menu.Menu;
import com.nonesole.web.platform.sys.menu.MenuTree;
import com.nonesole.web.platform.sys.rightset.IRightSet;
import com.nonesole.web.platform.sys.rightset.RightSetAction;
import com.nonesole.web.platform.sys.rightset.RightSetEnum;
import com.nonesole.web.platform.sys.rightset.RoleMenu;
import com.nonesole.web.platform.sys.user.UserEnum;

/**
 * As the core of platform, ActionControler has five effects:<br/>
 * 1. ActionController works under MVC patterns.<br/>
 * 2. Used to manage singleton objects of registered actions which depend on LocalTread.<br/>
 * 3. Used to call ActionControler object which depends on LocalTread.<br/>
 * 4. As the only one entrance, ActionControler is used to manage all business actions.<br/>
 * 5. Used to deal with problems for login security.<br/><br/>
 * 
 * Roya does not use AOP and IoC solution, because it is just a micro-core platform.
 * The management of business actions and views need depend on the database, but not the XML files.
 * The DAO layer of Roya is managed by Persistence2 framework.
 */
@WebServlet(name = "action", urlPatterns = { "/action" })
public class ActionControler extends HttpServlet {
		
	public static final long serialVersionUID = 1L;
	
	public static final int ADMIN_ROLE_ID = 1; 
	
	public static Map<Class<?>,String> interface2function;
	public static Map<String,Class<?>> interfaceMap;
	public static Map<String,Class<?>> instanceMap;
	public static Map<String,ThreadLocal<Object>> threadlocalMap;
	public static Map<String,String> function2ObjectMap;
	public static Map<String,String> function2MethodMap;
    public static ThreadLocal<ActionControler>  controlerTread;
    private Logger logger;

	public static Map<String,Integer> url2RoleIdMap;
	public static Map<String,Integer> actionParameter2RoleIdRelationMap;
		
	public static Map<Integer,Menu> allMenuMap = null;
	public static List<Menu> allMenuList = null;
	public static Map<Integer,List<Integer>> allRoleMenuMap = null;
	
	public static Map<String,String> commonBackPages = null;
	public static Map<String,String> adminBackPages = null;
	
	private static Map<Integer,Integer> lockedUserMap = null;
	
	public static String rightsetMenuString = null;
	
    /**
     * @see HttpServlet#HttpServlet()
     */
    public ActionControler() {
        super();
        logger = Logger.getInstance();
        if ( interfaceMap == null || 
        	instanceMap == null ||
        	threadlocalMap == null ||
        	function2ObjectMap == null ||
        			interface2function == null ) {
        	init();
        }
    }

    /**
     * Initialize the relations of user, roles, menus, functions, views.
     */
    public void init(){
    	interfaceMap = new HashMap<String,Class<?>>();
    	instanceMap = new HashMap<String,Class<?>>();
    	threadlocalMap = new HashMap<String,ThreadLocal<Object>>();
    	function2ObjectMap = new HashMap<String,String>();
    	function2MethodMap = new HashMap<String,String>();
    	commonBackPages = new HashMap<String,String>();
    	adminBackPages = new HashMap<String,String>();
    	lockedUserMap = new HashMap<Integer,Integer>();
    	interface2function = new HashMap<Class<?>,String>();
    			
    	try {
			this.initAllDefinedCache();
		} catch ( ExecuteSQLException e ) {
			logger.error("Definded function config has an error.", e);
		}
    	
		try {
			PlatformXMLParser pxp = new PlatformXMLParser();
			pxp.init();
		} catch (Exception e) {
			logger.error("There is an error on parsing roya.xml.", e);
		}
    	
    	try {
			this.initAccessControlCache();
		} catch (CoreException | InstantiationException | IllegalAccessException e) {
			logger.error("Access control setting has an error.", e);
		}
    	
    	try {
			this.initMenuCache();
		} catch (CoreException e) {
			logger.error("To initial menu cache has an error.", e);
		}
    
    }
    
    /**
     * @param pageName
     */
    public void setCommonPageUrl(String pageName){
    	commonBackPages.put(PlatformTools.getPathForRoot(pageName), pageName);
    }
    
    /**
     * @param pageName
     */
    public void setAdminBackPageUrl(String pageName){
    	adminBackPages.put(PlatformTools.getPathForRoot(pageName), pageName);
    }
    
    /**
     * @return
     */
    public static ActionControler getInstance(){
    	if ( controlerTread == null ) {
    		controlerTread = new ThreadLocal<ActionControler>();
    	}
    	ActionControler ctrl = controlerTread.get();
    	if ( null == ctrl ) {
    		ctrl = new ActionControler();
    		controlerTread.set(ctrl);
		}
		return controlerTread.get();
    }
    
    /**
	 * 调用方式:ActionControler.getInstanc().doAction(request,response);<br/>
	 * 
	 * This method is used to find business action by the name from request and 
	 * execute the action when the other action in JSP or Servlet call it.<br/>
	 * Example: <br/>
	 * request.setAttribute(ParameterEnum.ACTION.toString(),IUser.SAVE_USER_FUNCTION);<br/>
	 * ActionControler.getInstanc().doAction(request,response);<br/>
	 * User u = (User)request.getAttribute(IUser.OBJECT);<br/>
	 * @param HttpServletRequest
	 * @param HttpServletResponse
	 */
	public void doAction(HttpServletRequest request, HttpServletResponse response) throws CoreException {
		Logger.getInstance().debug("doAction", null);
		try {
			request.setCharacterEncoding("utf-8");
		} catch (UnsupportedEncodingException e1) {
			throw new CoreException("Can not set character.",e1.fillInStackTrace());
		}
		Object a = request.getAttribute(ParameterEnum.ACTION.toString());
		
		String functionId = "";
		if ( !this.isNull(a)) {
			functionId = a.toString();
		} else {
			throw new CoreException("Function id is null.");
		}
		try {
			Object actionObject = this.getActionByFunction(functionId);
			if ( actionObject != null ) {
				String methodName = function2MethodMap.get(functionId);
				if ( !isNull(methodName) ) {
					Class<?> clazz = instanceMap.get(function2ObjectMap.get(functionId));
					Method m = clazz.getMethod(methodName,
							new Class[]{HttpSession.class,HttpServletRequest.class,HttpServletResponse.class});
					m.invoke(actionObject, new Object[]{request.getSession(),request,response});
				}
			}
		} catch (InstantiationException | IllegalAccessException 
				| NoSuchMethodException | SecurityException 
				| IllegalArgumentException | InvocationTargetException e) {
			logger.debug("ActionControler:doAction", e);
		}
		
	}
    
	/**
	 * @see Servlet#init(ServletConfig)
	 */
	public void init(ServletConfig config) throws ServletException {
		System.out.println("factory init");
	}

	/**
	 * @see Servlet#destroy()
	 */
	public void destroy() {
	}

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		request.setCharacterEncoding("utf-8");
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		System.out.println("doPost");
		request.setCharacterEncoding("utf-8");
		String functionId = request.getParameter(ParameterEnum.ACTION.toString());
		try {
			Object actionObject = this.getActionByFunction(functionId);
			HttpSession session = request.getSession();
			if ( actionObject != null ) {
				String methodName = function2MethodMap.get(functionId);
				if ( !isNull(methodName) ) {
					Class<?> clazz = instanceMap.get(function2ObjectMap.get(functionId));
					Method m = clazz.getMethod(methodName,
							new Class[]{HttpSession.class,HttpServletRequest.class,HttpServletResponse.class});
					m.invoke(actionObject, new Object[]{session,request,response});
				}
			}
		} catch (InstantiationException | IllegalAccessException 
				| NoSuchMethodException | SecurityException 
				| IllegalArgumentException | InvocationTargetException e) {
			logger.error("ActionControler:doPost", e);
		}
		
	}

	
	/**
	 * Get Action object by function id
	 * @return Object
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	protected Object getActionByFunction(String functionId) 
			throws InstantiationException, IllegalAccessException {
		String actionName = function2ObjectMap.get(functionId);
		Logger.getInstance().debug("function : " + functionId + " , actionName : " + actionName , null);
		if ( isNull(actionName) ) {
			return null;
		}
		
		ThreadLocal<Object> tl = threadlocalMap.get(actionName);
		if ( isNull(tl) ) {
			return null;
		}
		
		if ( null == tl.get() ) {
			Class<?> clazz = instanceMap.get(actionName);
			if ( isNull(clazz) ) {
				return null;
			}
			tl.set(clazz.newInstance());
		}
		
		return tl.get();
	}
	
	/**
	 * Get Action object by name
	 * @return Object
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	protected Object getAction(String actionName) 
			throws InstantiationException, IllegalAccessException {
		logger.debug("actionName : " + actionName , null);
		if ( isNull(actionName) ) {
			return null;
		}
		
		ThreadLocal<Object> tl = threadlocalMap.get(actionName);
		if ( isNull(tl) ) {
			return null;
		}
		
		if ( null == tl.get() ) {
			Class<?> clazz = instanceMap.get(actionName);
			if ( isNull(clazz) ) {
				return null;
			}
			tl.set(clazz.newInstance());
		}
		
		return tl.get();
	}
	
	/**
	 * Get Action object by name and class
	 * @param theCaller 
	 * @param actionName 
	 * @return Object
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public Object getRegistrationAction(String theCaller,Class<?> interfaceClass) 
			throws Exception {
		if ( isNull(theCaller) || isNull(interfaceClass) ) {
			return null;
		}
		String actionName = interface2function.get(interfaceClass);
		logger.info("System action "+ actionName +" will be called by " + theCaller +".", null);
		if ( isNull(actionName) ) {
			return null;
		}
		return this.getAction(actionName);
		
	}
	
	/**
	 * Check whether the object is null.
	 * @param s
	 * @return
	 */
	protected boolean isNull(Object s){
		if ( null == s ) {
			return true;
		} else {
			if ( s instanceof String ) {
				if ( "".equals(s) ) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Initialize all the actions' cache which depend on ThreadLocal
	 * @throws ExecuteSQLException
	 * @throws ClassNotFoundException
	 */
	public void initAllDefinedCache() throws ExecuteSQLException{
		@SuppressWarnings("unchecked")
		IPojoAction<Function> pAction = (IPojoAction<Function>) PersistenceFactory.getInstance().getPojoAction();
		List<Function> objs = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_FUNCTIONS.toString(),null);
		if ( objs != null && !objs.isEmpty() ) {
			for ( Function obj : objs ) {
				String actionName = null;
				if ( obj.validate(Function.SYS_LOAD) ) {
					actionName = obj.getClass_name();
					try {
						interfaceMap.put(actionName, Class.forName(obj.getInterface_name()));
						instanceMap.put(actionName, Class.forName(obj.getClass_name()));
						threadlocalMap.put(actionName, new ThreadLocal<Object>());
						interface2function.put(Class.forName(obj.getInterface_name()), actionName);
						function2ObjectMap.put(obj.getParameter_name(), actionName);
						function2MethodMap.put(obj.getParameter_name(), obj.getMethod_name());
					} catch (ClassNotFoundException e) {
						Logger.getInstance().error("Definded function config has an error.", e);
						continue;
					}
					
				}
			}
		}
	}
	
	/**
	 * Initialize the parameters for access-control
	 * @throws CoreException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	public void initAccessControlCache() 
			throws CoreException, InstantiationException, IllegalAccessException {
		IRightSet action = (RightSetAction)this.getAction(RightSetEnum.ACTION_RIGHT_SET.toString());
		url2RoleIdMap = action.getUrl2RoleIdRelation();
		actionParameter2RoleIdRelationMap = action.getActionParameter2RoleIdRelation();
		
		if ( url2RoleIdMap == null ) {
			url2RoleIdMap = new HashMap<String,Integer>();
		}		
		
		if ( actionParameter2RoleIdRelationMap == null ) {
			actionParameter2RoleIdRelationMap = new HashMap<String,Integer>();
		}
		if ( function2ObjectMap != null ) {
			Set<String> set = function2ObjectMap.keySet();
			for ( String key : set ) {
				this.setActionParameter2RoleIdRelation(key);
			}
		}
		
	}
	
	/**
	 * @param parameter
	 */
	protected void setActionParameter2RoleIdRelation(String parameter){
		actionParameter2RoleIdRelationMap.put(parameter, ADMIN_ROLE_ID);
	}
	
	/**
	 * Initialize all the menus
	 * @throws CoreException
	 */
	public void initMenuCache() throws CoreException{
		logger.debug("ActionControler:initMenuCache", null);
		try {
			@SuppressWarnings("unchecked")
			IPojoAction<Menu> pAction = (IPojoAction<Menu>) PersistenceFactory.getInstance().getPojoAction();
			List<Menu> objs = pAction.pojoListAction(RightSetEnum.DAO_FIND_ALL_MENUS.toString(),null);
			allMenuList = objs;
			if ( objs != null && !objs.isEmpty() ) {
				allMenuMap = new HashMap<Integer,Menu>();
				for ( Menu obj : objs ) {
					allMenuMap.put(obj.getId(), obj);
				}
			} 
			
			@SuppressWarnings("unchecked")
			IPojoAction<RoleMenu> rmAction = (IPojoAction<RoleMenu>) PersistenceFactory.getInstance().getPojoAction();
			List<RoleMenu> rms = rmAction.pojoListAction(RightSetEnum.DAO_GET_ALL_ROLE_MENU.toString(),null);
			if ( rms != null && !rms.isEmpty() ) {
				allRoleMenuMap = new HashMap<Integer,List<Integer>>();
				List<Integer> menuId = null;
				for ( RoleMenu obj : rms ) {
					menuId = allRoleMenuMap.get(obj.getRole_id());
					if ( menuId == null ) {
						menuId = new ArrayList<Integer>();
						allRoleMenuMap.put(obj.getRole_id(), menuId);
					}
					menuId.add(obj.getMenu_id());
				}
			} 
			
		} catch (ExecuteSQLException e) {
			logger.error("To get relation of url2RoleId has error.", e);
		}		
	}
	

	/**
	 * Get menu for login user.
	 * @param request
	 * @return
	 */
	public MenuTree getUserMenu(HttpServletRequest request){
		Object l = request.getSession().getAttribute(UserEnum.ATTR_SESSION_USER_ROLES.toString());
		if ( l == null ) {
			return null;
		} 
		@SuppressWarnings("unchecked")
		List<Integer> roles = (List<Integer>)l;
		if ( roles.isEmpty() ) {
			return null;
		}
		
		Set<Integer> menuSet = new HashSet<Integer>();
		List<Integer> r = null;
		if ( roles != null && allRoleMenuMap != null && allMenuMap != null  ) {
			for ( int role : roles ) {
				if ( role > 1 ) {
					r = allRoleMenuMap.get(role);
					if ( r != null ) {
						menuSet.addAll(r);
					}
				}
			}
			r = new ArrayList<Integer>();
			r.addAll(menuSet);
			try {
				MenuTree mt = new MenuTree(allMenuList,r);
				return mt;
			} catch (CoreException e) {
				Logger.getInstance().error("To create menu tree has error.", e);
			}
			
		}
		return null;
	}
	
	/**
	 * Add user's id to user-locked cache.
	 * @param user
	 */
	public void addLockedUser(int id) {
		if ( id > 0 ) {
			if ( lockedUserMap == null ) {
				lockedUserMap = new HashMap<Integer,Integer>();
			}
			lockedUserMap.put(id, id);
		}
	}
	
	/**
	 * Release user's id from user-locked cache.
	 * @param user
	 */
	public void removeLockedUser(int id) {
		if ( id > 0 ) {
			if ( lockedUserMap == null ) {
				lockedUserMap = new HashMap<Integer,Integer>();
			}
			lockedUserMap.remove(id);
		}
	}
	
	/**
	 * Check whether the user is locked.
	 * @param id
	 * @return
	 */
	public boolean isUserLocked(int id) {
		if ( id > 0 ) {
			if ( lockedUserMap != null ) {
				return lockedUserMap.get(id)==null?false:true;
			}
		}
		return false;
	}
}
