/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.access;

import java.io.IOException;
import java.net.URI;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IMapAction;
import com.nonesole.persistence2.sql.define.IOrmAction;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.define.ISingleValueAction;
import com.nonesole.persistence2.sql.define.IVoidAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.basic.BasicAction;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.pages.Condition;
import com.nonesole.web.platform.pages.Pages;
import com.nonesole.web.platform.sys.CoreException;
import com.nonesole.web.platform.sys.PlatformTools;
import com.nonesole.web.platform.sys.userdetail.UserDetailEnum;

/**
 * Implementation class of IAccess
 * @author jack lee
 * @version 1.0 - build on 2017-01-30
 */
public class AccessAction extends BasicAction implements IAccess{

	private long dayTime = 0l;
	
	public AccessAction(){
		super();
		dayTime = 24 * 60 *60 *1000;
	}
	
	@Override
	public void add(HttpSession session, HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		try {
			AccessInfo info = (AccessInfo) request.getAttribute(IAccess.BEAN_NAME);
			if ( !info.validate(IAccess.INSERT) ) {
				logger.error("AccessInfo object has wrong fields.Please check it("+info+")", null);
				return;
			}
			IOrmAction action = PersistenceFactory.getInstance().getOrmAction();
			action.insert(info);
			logger.info("AccessInfo id for save : " + info.getId(), null);
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:add", e);
		}
	}

	@Override
	public void setLeaveTime(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		//logger.debug("AccessAction:setLeaveTime", null);
		try {
			Map<String,Object> map = this.parameter2map(IAccess.DAO_SET_LEAVE_TIME, request);
			IVoidAction action = PersistenceFactory.getInstance().getVoidAction();
			action.voidAction(IAccess.DAO_SET_LEAVE_TIME, map);
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:setLeaveTime", e);
		}
	}

	@Override
	public void sourceAnalysis(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		String strDateType = request.getParameter(IAccess.PARAM_SOURCE_DATE_TYPE);
		int dateType = PlatformTools.toInt(strDateType);
		long sTime,eTime;
		try {
			if ( dateType == 0 || dateType == -1 ) {
				sTime = this.getDayStartTime(dateType);
				eTime = this.getDayEndTime(dateType);
			} else {
				sTime = this.getDaysIntervalStartTime(dateType);
				eTime = this.getDaysIntervalEndTime(dateType);
			}
			
		} catch (ParseException e) {
			logger.error("Can not get the time of day.", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_1"));
			return;
		}
		
		Map<String,Object> map = new HashMap<String,Object>();
		map.put(IAccess.SOURCE_START_TIME, sTime);
		map.put(IAccess.SOURCE_END_TIME, eTime);
		IMapAction action = PersistenceFactory.getInstance().getMapAction();
		
		List<Map<String,Object>> list = null;
		try {
			if ( dateType < 1 ) {
				list = action.mapListAction(IAccess.DAO_FIND_SOURCE_BY_HOURS, map);
			} else {
				list = action.mapListAction(IAccess.DAO_FIND_SOURCE_BY_DAYS, map);
			}
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:sourceAnalysis", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_1"));
			return;
		}
		
		int linksPv = 0, totalPv = 0, enginesPv = 0, directsPv = 0;
		Map<String,Object> linksMap = new HashMap<String,Object>(), enginesMap, directsMap;
		List<Map<String,Object>> links = null;
		try {
			links = action.mapListAction(IAccess.DAO_FIND_SOURCE_DOMAIN, map);
			linksMap = this.countPvPercent(links);
			if ( PlatformTools.isNull(linksMap) ) {
				linksMap = new HashMap<String,Object>();
			}
			linksPv = this.obj2Int(linksMap.get(IAccess.PV));
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:sourceAnalysis", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_1"));
			return;
		}
		
		List<Map<String,Object>> engines = null;
		try {
			engines = action.mapListAction(IAccess.DAO_FIND_SOURCE_SEARCH_ENGINE, map);
			enginesMap = this.countPvPercent(engines);
			if ( PlatformTools.isNull(enginesMap) ) {
				enginesMap = new HashMap<String,Object>();
			}
			enginesPv = PlatformTools.isNull(enginesMap.get(IAccess.PV))?0:(Integer)enginesMap.get(IAccess.PV);
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:sourceAnalysis", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_1"));
			return;
		}
		
		List<Map<String,Object>> directs = null;
		try {
			directs = action.mapListAction(IAccess.DAO_FIND_SOURCE_DIRECT, map);
			directsMap = this.countPvPercent(directs);
			if ( PlatformTools.isNull(directsMap) ) {
				directsMap = new HashMap<String,Object>();
			}
			directsPv = PlatformTools.isNull(directsMap.get(IAccess.PV))?0:(Integer)directsMap.get(IAccess.PV);
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:sourceAnalysis", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_1"));
			return;
		}
		
		Map<String,Object> totalMap = new HashMap<String,Object>();
		totalPv = linksPv + enginesPv + directsPv;
		totalMap.put(IAccess.PV, totalPv==0?1:totalPv);
		double tpv = Double.valueOf(totalPv);
		linksMap.put(IAccess.PV_PERCENT, PlatformTools.double2PercentString2(linksPv/tpv));
		enginesMap.put(IAccess.PV_PERCENT, PlatformTools.double2PercentString2(enginesPv/tpv));
		directsMap.put(IAccess.PV_PERCENT, PlatformTools.double2PercentString2(directsPv/tpv));
		
		int totalOnlyOnePv = this.obj2Int(linksMap.get(IAccess.ONLY_ONE_PV)) +
				                     this.obj2Int(enginesMap.get(IAccess.ONLY_ONE_PV)) +
				                     this.obj2Int(directsMap.get(IAccess.ONLY_ONE_PV));
		
		double topv = tpv==0?0d:totalOnlyOnePv/tpv;
		totalMap.put(IAccess.ONLY_ONE_PV_PERCENT, PlatformTools.double2PercentString2(topv));
		
		double totalStayTime = (this.obj2Double(linksMap.get(IAccess.STAY_TIME)) + 
							    this.obj2Double(enginesMap.get(IAccess.STAY_TIME)) + 
							    this.obj2Double(directsMap.get(IAccess.STAY_TIME)) ) / 3;
		
		totalMap.put(IAccess.STAY_TIME, PlatformTools.double2String(totalStayTime));
		
		ISingleValueAction sAction = PersistenceFactory.getInstance().getSingleValueAction();
		try {
			int totalUv = sAction.intAction(IAccess.DAO_FIND_SOURCE_TOTAL_UV, map);
			totalMap.put(IAccess.UV, totalUv);
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:sourceAnalysis", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_1"));
			return;
		}
		
		EchartBean eb = new EchartBean();
		eb.setTitleText(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_31"));
		if ( dateType == IAccess.TODAY ) {
			eb.setTitleSubtext(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_2"));
			eb.setxAxisData(this.getXAxisData(1));
		} else if ( dateType == IAccess.YESTODAY ) {
			eb.setTitleSubtext(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_3"));
			eb.setxAxisData(this.getXAxisData(1));
		} else if ( dateType == IAccess.LAST_7_DAYS ) {
			eb.setTitleSubtext(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_4"));
			eb.setxAxisData(this.getXAxisData(7));
		} else if ( dateType == IAccess.LAST_30_DAYS ) {
			eb.setTitleSubtext(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_5"));
			eb.setxAxisData(this.getXAxisData(30));
		} 
		eb.setyAxisFormatter("{value} "+LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_6"));
		eb.setLegendData("['"+ LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_7") +"',"
						+ "'"+ LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_8") +"',"
						+ "'"+ LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_9") +"']");
		if ( list == null || list.isEmpty() ) {
			List<EchartSeries> l = new ArrayList<EchartSeries>();
			l.add(this.getDefaultEchartSeries(dateType, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_7")));
			l.add(this.getDefaultEchartSeries(dateType, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_8")));
			l.add(this.getDefaultEchartSeries(dateType, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_9")));
			eb.setSeries(l);
		} else {
			try {
				eb.setSeries(this.list2EchartSeries(request, dateType, list));
			} catch (ParseException e) {
				logger.error("AccessAction:sourceAnalysis", e);
				PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_1"));
				return;
			}
		}
		
		request.setAttribute(IAccess.ATTR_ECHART_BEN, eb);
		request.setAttribute(IAccess.ATTR_TOTAL_DATA, totalMap);
		request.setAttribute(IAccess.ATTR_LINKS_DATA, linksMap);
		request.setAttribute(IAccess.ATTR_ENGINES_DATA, enginesMap);
		request.setAttribute(IAccess.ATTR_DIRECTS_DATA, directsMap);
		
		request.setAttribute(IAccess.ATTR_LINKS_DETAIL, links);
		request.setAttribute(IAccess.ATTR_ENGINES_DETAIL, engines);
		request.setAttribute(IAccess.ATTR_DIRECTS_DETAIL, directs);
		
	}
	
	/**
	 * @param map
	 * @return
	 */
	protected String getOnlyOnePvPercent(Map<String,Object> map){
		if ( map == null ) {
			return "0.00%";
		}
		int pv = this.obj2Int(map.get(IAccess.PV));
		int opv = this.obj2Int(map.get(IAccess.ONLY_ONE_PV));
		double opvp = opv==0?0d:pv/opv;
		return PlatformTools.double2PercentString2(opvp);
	}
	
	/**
	 * @param dateType
	 * @param name
	 * @return
	 * @throws ParseException 
	 */
	private List<EchartSeries> list2EchartSeries(HttpServletRequest request, int dateType, List<Map<String,Object>> list) throws ParseException{
		List<EchartSeries> esList = new ArrayList<EchartSeries>();
		if ( list == null || list.isEmpty() )
			return esList;
		
		String[] key = new String[]{LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_7"),
				LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_8"),
				LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_9")};
		Integer[] type = new Integer[]{0,1,2};
		Map<String,Object> map = null;
		List<Map<String,Object>> links = new ArrayList<Map<String,Object>>();
		List<Map<String,Object>> engines = new ArrayList<Map<String,Object>>();
		List<Map<String,Object>> directs = new ArrayList<Map<String,Object>>();
		
		for ( int i = 0 ; i < list.size() ; i++ ) {
			map = list.get(i);
			if ( type[0].equals(map.get(IAccess.SOURCE_TYPE)) ) {
				links.add(map);
			} else if ( type[1].equals(map.get(IAccess.SOURCE_TYPE)) ) {
				engines.add(map);
			} else if ( type[2].equals(map.get(IAccess.SOURCE_TYPE)) ) {
				directs.add(map);
			}
		}
		
		if ( dateType == IAccess.TODAY || dateType == IAccess.YESTODAY ) {
			esList.add(this.getHoursEchartSeries(links, key[0]));
			esList.add(this.getHoursEchartSeries(engines, key[1]));
			esList.add(this.getHoursEchartSeries(directs, key[2]));
		} else if ( dateType == IAccess.LAST_7_DAYS )  {
			esList.add(this.getDaysEchartSeries(links, key[0], 7));
			esList.add(this.getDaysEchartSeries(engines, key[1], 7));
			esList.add(this.getDaysEchartSeries(directs, key[2], 7));
		} else if ( dateType == IAccess.LAST_30_DAYS )  {
			esList.add(this.getDaysEchartSeries(links, key[0], 30));
			esList.add(this.getDaysEchartSeries(engines, key[1], 30));
			esList.add(this.getDaysEchartSeries(directs, key[2], 30));
		}
		return esList;
	}
	
	/**
	 * @param l
	 * @param name
	 * @return
	 * @throws ParseException 
	 */
	private EchartSeries getDaysEchartSeries(List<Map<String,Object>> l, String name, int days) throws ParseException{
		Object time = null;
		boolean existTime = false;
		Object number = null;
		Map<String,Object> map = null;
		String key = null;
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		for ( int i = days-1 ; i > -1 ; i-- ) {
			existTime = false;
			key = PlatformTools.getFormatTime(this.getDaysIntervalStartTime(i), "yyyy-MM-dd");
			for ( int j = 0 ; j < l.size() ; j++ ) {
				map = l.get(j);
				time = map.get(IAccess.TIME);
				if ( key.equals(time) ) {
					number = map.get(IAccess.NUMBER);
					if ( number == null || "".equals(number) ) {
						sb.append("0");
					} else {
						sb.append((Integer)number);
					}
					existTime = true;
					break;
				}
			}
			if ( !existTime ) {
				sb.append("0");
			}
			if ( i > 0 ) {
				sb.append(",");
			}
		}
		sb.append("]");
		
		EchartSeries es = new EchartSeries();
		es.setName(name);
		es.setData(sb.toString());
		return es;
	}
	
	/**
	 * @param l
	 * @param name
	 * @return
	 */
	private EchartSeries getHoursEchartSeries(List<Map<String,Object>> l, String name){
		Object time = null;
		boolean existTime = false;
		Object number = null;
		Map<String,Object> map = null;
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		for ( int i = 0 ; i < 25 ; i++ ) {
			for ( int j = 0 ; j < l.size() ; j++ ) {
				map = l.get(j);
				time = map.get(IAccess.TIME);
				if ( time != null && !"".equals(time) ) {
					if ( i == Integer.valueOf(String.valueOf(time)) ) {
						number = map.get(IAccess.NUMBER);
						if ( number == null || "".equals(number) ) {
							sb.append("0");
						} else {
							sb.append((Integer)number);
						}
						existTime = true;
						break;
					}
				}
			}
			if ( !existTime ) {
				sb.append("0");
			}
			if ( i < 24 ) {
				sb.append(",");
			}
			existTime = false;
		}
		sb.append("]");
		
		EchartSeries es = new EchartSeries();
		es.setName(name);
		es.setData(sb.toString());
		return es;
	}
	
	/**
	 * @param dateType
	 * @param name
	 * @return
	 */
	private EchartSeries getDefaultEchartSeries(int dateType, String name){
		EchartSeries es1 = new EchartSeries();
		es1.setName(name);
		if ( dateType == IAccess.TODAY || dateType == IAccess.YESTODAY ) {
			es1.setData(this.getDefaultSeriesData(1));
		} else if ( dateType == IAccess.LAST_7_DAYS )  {
			es1.setData(this.getDefaultSeriesData(7));
		} else if ( dateType == IAccess.LAST_30_DAYS )  {
			es1.setData(this.getDefaultSeriesData(30));
		}
		return es1;
	}
	
	/**
	 * @param days
	 * @return
	 */
	private String getDefaultSeriesData(int days){
		if ( days == 1 ) {
			return "[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]";
		} else {
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			for ( int i = 0 ; i < days ; i++ ) {
				sb.append("0");
				if ( i < days-1 ) {
					sb.append(",");
				}
			}
			sb.append("]");
			return sb.toString();
		}
	}
	
	
	
	/**
	 * @param days
	 * @return 
	 */
	private String getXAxisData(int days){
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		if ( days == 1 ) {
			for ( int i = 0 ; i < 24 ; i++ ) {
				sb.append("'");
				sb.append(i).append("h");
				sb.append("'");
				if ( i < 23 ) {
					sb.append(",");
				}
			}
		} else {
			int d = -1 * days + 1;
			String day = null;
			String format = "MM-dd";
			long datetime = System.currentTimeMillis();
			
			for ( int i = d ; i < 1 ; i++ ) {
				day = this.getDate(format, datetime, i);
				sb.append("'");
				sb.append(day);
				sb.append("'");
				if ( i < days ) {
					sb.append(",");
				}
			}
		}
		sb.append("]");
		return sb.toString();
	}
	
	/**
	 * @param list
	 * @return Map<String,Object>
	 */
	private Map<String,Object> countPvPercent(List<Map<String,Object>> list){
		if ( list == null || list.isEmpty() ) {
			return null;
		}
		Map<String,Object> totalMap = new HashMap<String,Object>();
		
		int pv = 0,onlyOnePv = 0,allOnlyOnePv = 0,allpv = 0,
				alluv = 0,uv = 0, ips = 0,allIps = 0,fpv = 0, allfpv = 0;
		double stayTime = 0.0d,allStayTime = 0.0d;
		int count = 0;
		for ( Map<String,Object> map : list ) {
			pv = this.obj2Int(map.get(IAccess.PV));
			onlyOnePv = this.obj2Int(map.get(IAccess.ONLY_ONE_PV));
			uv = this.obj2Int(map.get(IAccess.UV));
			stayTime = this.obj2Double(map.get(IAccess.STAY_TIME));
			ips = this.obj2Int(map.get(IAccess.IP));
			fpv = this.obj2Int(map.get(IAccess.FIRST_PV));
			
			allOnlyOnePv += onlyOnePv;
			allpv += pv;
			alluv += uv;
			allStayTime += stayTime;
			allIps += ips;
			allfpv += fpv;
			count++;
			
			if ( pv < 1 ) {
				pv = 1;
			}
			
			map.put(IAccess.ONLY_ONE_PV_PERCENT, PlatformTools.double2PercentString(onlyOnePv/pv));
		}
		if ( allpv < 1 )
			allpv = 1;
		totalMap.put(IAccess.PV, allpv);
		totalMap.put(IAccess.UV, alluv);
		totalMap.put(IAccess.STAY_TIME, allStayTime/count);//由于要在外面汇总数据，因此这里不进行转换
		totalMap.put(IAccess.ONLY_ONE_PV_PERCENT, PlatformTools.double2PercentString(allOnlyOnePv/allpv));
		totalMap.put(IAccess.ONLY_ONE_PV, allOnlyOnePv);
		totalMap.put(IAccess.IP, allIps);
		totalMap.put(IAccess.FIRST_PV, allfpv);
		
		for ( Map<String,Object> map : list ) {
			pv = this.obj2Int(map.get(IAccess.PV));
			map.put(IAccess.PV_PERCENT, PlatformTools.double2PercentString(pv/allpv));
		}
		return totalMap;
	}
	
	/**
	 * @param obj
	 * @return
	 */
	private Integer obj2Int(Object obj){
		return PlatformTools.isNull(obj)?0:(Integer)obj;
	}
	
	/**
	 * @param obj
	 * @return
	 */
	private Double obj2Double(Object obj){
		return PlatformTools.isNull(obj)?0.0d:(Double)obj;
	}
	
	/**
	 * @return 
	 * @throws ParseException
	 */
	private long todayStartTime() throws ParseException{
		String model = "yyyy-MM-dd"; //yyyy/MM/dd HH:mm:ss.SSS
		SimpleDateFormat formater = new SimpleDateFormat(model);
		String date = formater.format(new Date(System.currentTimeMillis()));
		date = date + " 00:00:00.000 ";
		model = "yyyy-MM-dd HH:mm:ss.SSS";
		formater = new SimpleDateFormat(model);
		Date newDate = formater.parse(date);
		return newDate.getTime();
	}
	
	/**
	 * @return 今天的起始时间
	 * @throws ParseException
	 */
	private long todayEndTime() throws ParseException{
		String model = "yyyy-MM-dd"; //yyyy/MM/dd HH:mm:ss.SSS
		SimpleDateFormat formater = new SimpleDateFormat(model);
		String date = formater.format(new Date(System.currentTimeMillis()));
		date = date + " 23:59:59.999 ";
		model = "yyyy-MM-dd HH:mm:ss.SSS";
		formater = new SimpleDateFormat(model);
		Date newDate = formater.parse(date);
		return newDate.getTime();
	}
	
	/**
	 * @param format
	 * @param datetime 
	 * @param days 
	 * @return
	 */
	private String getDate(String format, long datetime, int days){
		format = PlatformTools.isNull(format)?"yyyy-MM-dd":format;
		SimpleDateFormat formater = new SimpleDateFormat(format);
		long newDatetime = datetime + days * this.dayTime;
		return formater.format(new Date(newDatetime));
	}
	
	/**
	 * @param days
	 * @return
	 * @throws ParseException
	 */
	private long getDayStartTime(int days) throws ParseException {
		long todayStartTime = this.todayStartTime();
		return todayStartTime + days * dayTime;
	}
	
	/**
	 * @param days
	 * @return
	 * @throws ParseException
	 */
	private long getDayEndTime(int days) throws ParseException {
		long todayEndTime = this.todayEndTime();
		return todayEndTime + days * dayTime;
	}

	/**
	 * @param days
	 * @return
	 * @throws ParseException
	 */
	private long getDaysIntervalStartTime(int days) throws ParseException {
		long todayStartTime = this.todayStartTime();
		return todayStartTime - (days * dayTime);
	}
	
	/**
	 * @param days
	 * @return
	 * @throws ParseException
	 */
	private long getDaysIntervalEndTime(int days) throws ParseException {
		long todayEndTime = this.todayEndTime();
		return todayEndTime;
	}
	
	@Override
	public void targetAnalysis(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		//logger.debug("AccessAction:targetAnalysis", null);
		
		String strResourceName = request.getParameter(IAccess.PARAM_RESOURCE_NAME);
		if ( PlatformTools.isNull(strResourceName) ) {
			strResourceName = "jsp";
		}
		int nameLength = strResourceName.length();
		
		String strDateType = request.getParameter(IAccess.PARAM_SOURCE_DATE_TYPE);
		int dateType = PlatformTools.toInt(strDateType);
		long sTime,eTime;
		try {
			if ( dateType == 0 || dateType == -1 ) {
				sTime = this.getDayStartTime(dateType);
				eTime = this.getDayEndTime(dateType);
			} else {
				sTime = this.getDaysIntervalStartTime(dateType);
				eTime = this.getDaysIntervalEndTime(dateType);
			}
		} catch (ParseException e) {
			logger.error("Can not get the time of day.", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_10"));
			return;
		}
		
		Map<String,Object> map = new HashMap<String,Object>();
		map.put(IAccess.SOURCE_START_TIME, sTime);
		map.put(IAccess.SOURCE_END_TIME, eTime);
		map.put(IAccess.RESOURCE, strResourceName);
		map.put(IAccess.RESOURCE_SIZE, nameLength);
		
		IMapAction action = PersistenceFactory.getInstance().getMapAction();
		List<Map<String,Object>> details = null;
		Map<String,Object> countMap = null;
		try {
			details = action.mapListAction(IAccess.DAO_FIND_URL_TARGET, map);
			countMap = this.countPvPercent(details);
			if ( PlatformTools.isNull(countMap) ) {
				countMap = new HashMap<String,Object>();
			}
			countMap.put(IAccess.STAY_TIME, 
					PlatformTools.double2String(
							this.obj2Double(countMap.get(IAccess.STAY_TIME))));
			
			List<Map<String,Object>> total = action.mapListAction(IAccess.DAO_FIND_TOTAL_URL_TARGET, map);
			if ( total != null && !total.isEmpty() ) {
				if ( total.get(0) != null && total.get(1) != null ) {
					int uv = PlatformTools.toInt(total.get(0).get(IAccess.UV));
					int ips = PlatformTools.toInt(total.get(1).get(IAccess.IP));
					countMap.put(IAccess.UV, uv);
					countMap.put(IAccess.IP, ips);
				}
			}
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:targetAnalysis", e);
			PlatformTools.setCookieMessage(response, LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_11"));
			return;
		}
	
		request.setAttribute(IAccess.ATTR_TARGET_TOTAL_DATA, countMap);
		request.setAttribute(IAccess.ATTR_TARGET_DETAIL, details);
	}

	@Override
	public void visitorAnalysis(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		String strDateType = request.getParameter(IAccess.PARAM_SOURCE_DATE_TYPE);
		int dateType = PlatformTools.toInt(strDateType);
		long sTime,eTime;
		try {
			if ( dateType == 0 || dateType == -1 ) {
				sTime = this.getDayStartTime(dateType);
				eTime = this.getDayEndTime(dateType);
			} else {
				sTime = this.getDaysIntervalStartTime(dateType);
				eTime = this.getDaysIntervalEndTime(dateType);
			}
		} catch (ParseException e) {
			logger.error("Can not get the time of day.", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_11"));
			return;
		}
		
		Map<String,Object> map = new HashMap<String,Object>();
		map.put(IAccess.SOURCE_START_TIME, sTime);
		map.put(IAccess.SOURCE_END_TIME, eTime);
		
		IMapAction action = PersistenceFactory.getInstance().getMapAction();
		List<Map<String,Object>> details = null;
		try {
			details = action.mapListAction(IAccess.DAO_FIND_VISITOR_PROPERTY, map);
		} catch (ExecuteSQLException e) {
			logger.error("AccessAction:targetAnalysis", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_11"));
			return;
		}
		
		EchartBean browserEchart = visitor2EchartBean(request, details, IAccess.BROWSER, dateType);
		EchartBean osEchart = visitor2EchartBean(request, details, IAccess.OS, dateType);
		EchartBean mobileEchart = visitor2EchartBean(request, details, IAccess.MOBILE, dateType);
		EchartBean countryEchart = visitor2EchartBean(request, details, IAccess.COUNTRY, dateType);
		EchartBean languageEchart = visitor2EchartBean(request, details, IAccess.LANGUAGE, dateType);
		
		request.setAttribute(IAccess.BROWSER, browserEchart);
		request.setAttribute(IAccess.OS, osEchart);
		request.setAttribute(IAccess.MOBILE, mobileEchart);
		request.setAttribute(IAccess.COUNTRY, countryEchart);
		request.setAttribute(IAccess.LANGUAGE, languageEchart);
	}
	
	/**
	 * @param dateType
	 * @return
	 */
	private String getDateTypeText(HttpServletRequest request, int dateType) {
		if ( dateType == 0 ) {
			return LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_12");
		} else if ( dateType == -1 ) {
			return LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_13");
		} else if ( dateType > 0 ) {
			return LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_14")+dateType+
				   LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_15");
		} 
		return "";
	}
	
	/**
	 * @param result
	 * @param datasort
	 * @return
	 */
	private EchartBean visitor2EchartBean(HttpServletRequest request, List<Map<String,Object>> result, String datasort, int dateType){
		if ( PlatformTools.isNull(datasort) || result == null ) {
			return null;
		}
		EchartBean eb = new EchartBean();
		
		eb.setTitleSubtext(this.getDateTypeText(request, dateType));
		if ( IAccess.OS.equals(datasort) ) {
			eb.setTitleText(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_16"));
			eb.setSeriesName(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_17"));
		} else if ( IAccess.BROWSER.equals(datasort) ) {
			eb.setTitleText(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_18"));
			eb.setSeriesName(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_19"));
		} else if ( IAccess.MOBILE.equals(datasort) ) {
			eb.setTitleText(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_20"));
			eb.setSeriesName(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_21"));
		} else if ( IAccess.COUNTRY.equals(datasort) ) {
			eb.setTitleText(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_22"));
			eb.setSeriesName(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_23"));
		} else if ( IAccess.LANGUAGE.equals(datasort) ) {
			eb.setTitleText(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_24"));
			eb.setSeriesName(LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_25"));
		} 
		
		if ( !result.isEmpty() ) {
			int maxNum = 0;
			int num = 0;
			String k = null;
			String v = null;
			StringBuilder key = new StringBuilder();
			StringBuilder keyValue = new StringBuilder();
			key.append("[");
			keyValue.append("[");
			for ( Map<String,Object> map : result ) {
				if ( datasort.equals(map.get(IAccess.DS)) ) {
					num = this.obj2Int(map.get(IAccess.NUM));
					maxNum=maxNum>num?maxNum:num;
					k=(String)(map.get(IAccess.PROP)==null?LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_26"):map.get(IAccess.PROP));
					v=(String)(map.get(IAccess.NUM)==null?"0":String.valueOf(map.get(IAccess.NUM)));
					if ( IAccess.MOBILE.equals(datasort) ) {
						if ( "1".equals(k) ) {
							k=LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_27");
						} else {
							k=LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_28");
						}
					}
					
					//'Direct access'
					key.append("'").append(k).append("'").append(",");
					//{value:335, name:'Direct access'},
					keyValue.append("{value:").append(v).append(",name:'").append(k).append("'},");
				}
			}
			key.replace(key.length()-1, key.length(), "");
			key.append("]");
			keyValue.replace(keyValue.length()-1, keyValue.length(), "");
			keyValue.append("]");
			
			eb.setMaxValue(maxNum);
			eb.setKeys(key.toString());
			eb.setKeyValue(keyValue.toString());
		}
		
		return eb;
	}

	@Override
	public void saveSetting(HttpSession session, HttpServletRequest request,
			HttpServletResponse response) throws CoreException {
		StringBuilder sb = new StringBuilder();
		String switchValue = request.getParameter(IAccess.SWITCH_VALUE);
		String res = request.getParameter(IAccess.RESOURCE);
		String os = request.getParameter(IAccess.OS);
		String borwser = request.getParameter(IAccess.BROWSER);
		String mobile = request.getParameter(IAccess.MOBILE);
		String searchEngine = request.getParameter(IAccess.SEARCH_ENGINE);
		
		sb.append(IAccess.SET_SWITCH).append("\r\n")
		  .append(switchValue==null?"close":switchValue).append("\r\n")
		  .append(IAccess.SET_RESOURCE).append("\r\n")
		  .append(res==null?"":res).append("\r\n")
		  .append(IAccess.SET_OS).append("\r\n")
		  .append(os==null?"":os).append("\r\n")
		  .append(IAccess.SET_BROWSER).append("\r\n")
		  .append(borwser==null?"":borwser).append("\r\n")
		  .append(IAccess.SET_MOBILE).append("\r\n")
		  .append(mobile==null?"":mobile).append("\r\n")
		  .append(IAccess.SET_SEARCH_ENGINE).append("\r\n")
		  .append(searchEngine==null?"":searchEngine).append("\r\n");
		
		try {
			URI uri = AccessStatistics.getSetURI();
			PlatformTools.deleteFile(uri);
			//logger.debug(uri.getPath(), null);
			PlatformTools.saveFileFromString(uri, sb.toString());
		} catch (IOException e) {
			logger.error("AccessAction:saveSetting", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_29"));
		}
	}

	@Override
	public void getAccessInfoDetail(HttpSession session,
			HttpServletRequest request, HttpServletResponse response)
			throws CoreException {
		//logger.debug("AccessAction:getAccessInfoDetail", null);
		
		Condition resource = new Condition().setParamKey(IAccess.TARGET_URL).makeLikeCondition(request);
		Condition os = new Condition().setParamKey(IAccess.OS).makeCommonCondition(request);
		Condition browser = new Condition().setParamKey(IAccess.BROWSER).makeCommonCondition(request);
		Condition mobile = new Condition().setParamKey(IAccess.MOBILE).makeCommonCondition(request);
		Condition searchEngine = new Condition().setParamKey(IAccess.SEARCH_ENGINE).makeCommonCondition(request);
		Condition userAgent = new Condition().setParamKey(IAccess.USER_AGENT).makeLikeCondition(request);
		Condition ip = new Condition().setParamKey(IAccess.PARAM_IP).makeLikeCondition(request);
		Condition startTime = new Condition().setParamKey(IAccess.START_TIME).makeSelfDefinedCondition(request, IAccess.VISIT_TIME , ">");
		Condition endTime = new Condition().setParamKey(IAccess.END_TIME).makeSelfDefinedCondition(request, IAccess.VISIT_TIME ,"<=");
		
		String pageStr = request.getParameter(ParameterEnum.PAGE.toString());
		Pages<List<AccessInfo>> pages = new Pages<List<AccessInfo>>(request);
		pages.setCurrentPage(PlatformTools.page2Int(pageStr));
		pages.setUrl(IAccess.URL_DETAIL);
		
		Map<String, String> params = new HashMap<String, String>();
		StringBuilder sb = new StringBuilder();
		Map<String, Object> c = new HashMap<String, Object>();//属性值与变量对应关系用Hashmap保存。
		
		boolean cNull = resource.isEmpty() && 
				os.isEmpty() && 
				browser.isEmpty() && 
				mobile.isEmpty() && 
				searchEngine.isEmpty() && 
				userAgent.isEmpty() && 
				ip.isEmpty() && 
				startTime.isEmpty() && 
				endTime.isEmpty();
		
		if (!cNull) {
			sb.append(" where ");
			
			resource.makePreparedSQLWithLeftLike(sb, params, c);
			os.makePreparedSQL(sb, params, c);
			browser.makePreparedSQL(sb, params, c);
			mobile.makePreparedSQL(sb, params, c);
			searchEngine.makePreparedSQL(sb, params, c);
			userAgent.makePreparedSQLWithLike(sb, params, c);
			ip.makePreparedSQLWithLike(sb, params, c);
			Long st = 0l;
			String stParam = startTime.getParamValue();
			try {
				st = PlatformTools.string2time(stParam, IAccess.VIEW_TIME_FORMAT);
			} catch (ParseException e) {
				logger.error("Error on parsing start time.", e);
			}
			startTime.setParamValue(st.toString());
			startTime.makePreparedSQL(sb, params, c);
			params.put(startTime.getParamKey(), stParam);
			
			Long et = 0l;
			String etParam = endTime.getParamValue();
			try {
				et = PlatformTools.string2time(etParam, IAccess.VIEW_TIME_FORMAT);
			} catch (ParseException e) {
				logger.error("Error on parsing end time.", e);
			}
			endTime.setParamValue(et.toString());
			endTime.makePreparedSQL(sb, params, c);
			params.put(endTime.getParamKey(), etParam);
			
			pages.setParams(params);
		}

		try {
			c.put(IAccess.CONDITION, sb.toString());
			ISingleValueAction sAction = PersistenceFactory.getInstance().getSingleValueAction();	
			Integer rows = sAction.intAction(IAccess.DAO_ALL_ROWS, c);
			if ( rows == null ) {
				rows = 0;
			}
			pages.setRows(rows);
			c.put(UserDetailEnum.PARAM_START_INDEX.toString(),pages.getStartNumber());
			c.put(UserDetailEnum.PARAM_PAGE_SIZE.toString(),pages.getPageSize());
			
			@SuppressWarnings("unchecked")
			IPojoAction<AccessInfo> acts = (IPojoAction<AccessInfo>) PersistenceFactory.getInstance().getPojoAction();
			List<AccessInfo> l = acts.pojoListAction(IAccess.DAO_FIND_ACCESS_INFO_DETAIL, c);
			if (l != null && !l.isEmpty()) {
				pages.setList(l);
			}
			request.setAttribute(IAccess.ATTR_ACCESS_DETAIL, pages);
			//System.out.println("query : " + ((UserDetail)l.get(0)).getId());

		} catch (ExecuteSQLException e) {
			logger.error("[AccessAction:getAccessInfoDetail]", e);
			PlatformTools.setCookieMessage(response, 
					LocaleToolkit.getLocaleString(request, "com.nonesole.web.platform.access.AccessAction_info_30"));
		}
		//this.redirect(response, IAccess.URL_DETAIL_2);
	}
	
	/**
	 * @param selectName 
	 * @param selectValue 
	 * @param keys 
	 * @param map 
	 * @return
	 */
	public static String vmSelectHtml(String selectName, String selectValue, 
			List<String> keys, Map<String,String> map){
		if ( PlatformTools.isNull(selectName) || keys == null || keys.isEmpty() 
				|| map == null || map.isEmpty() ) {
			return "";
		}
		String value = null;
		StringBuilder sb = new StringBuilder();
		sb.append("<select name=\"").append(selectName).append("\" class=\"input-sm input_width_1\">");
		sb.append("<option value=\"\">&nbsp;</option>").append("\r\n");
		for ( String key : keys) {
			value = map.get(key);
			if ( PlatformTools.isNull(key) || PlatformTools.isNull(value) ) {
				continue;
			} else {
				sb.append("<option value=\"").append(value).append("\"");
				if ( !PlatformTools.isNull(selectValue) ) {
					if ( value.equals(selectValue) ) {
						sb.append(" selected=\"selected\" ");
					}
				}
				sb.append(">")
				  .append(value)
				  .append("</option>")
				  .append("\r\n");
			}
		}
		sb.append("</select>");
		return sb.toString();
	}
	
	/**
	 * @param selectName 
	 * @param selectValue 
	 * @param values
	 * @return
	 */
	public static String vlSelectHtml(String selectName, String selectValue, List<String> values){
		if ( PlatformTools.isNull(selectName) || values == null || values.isEmpty() ) {
			return "";
		}
		StringBuilder sb = new StringBuilder();
		sb.append("<select name=\"").append(selectName).append("\" class=\"input-sm input_width_1\">");
		sb.append("<option value=\"\">&nbsp;</option>").append("\r\n");
		for ( String value : values) {
			if (  PlatformTools.isNull(value) ) {
				continue;
			} else {
				sb.append("<option value=\"").append(value).append("\"");
				if ( !PlatformTools.isNull(selectValue) ) {
					if ( value.equals(selectValue) ) {
						sb.append(" selected=\"selected\" ");
					}
				}
				sb.append(">")
				  .append(value)
				  .append("</option>")
				  .append("\r\n");
			}
		}
		sb.append("</select>");
		return sb.toString();
	}
}
