/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRegistration;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.nonesole.log.Logger;
import com.nonesole.persistence2.sql.PersistenceFactory;
import com.nonesole.persistence2.sql.define.IListValueAction;
import com.nonesole.persistence2.sql.define.IPojoAction;
import com.nonesole.persistence2.sql.exception.ExecuteSQLException;
import com.nonesole.web.platform.ActionControler;
import com.nonesole.web.platform.ParameterEnum;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.safe.SafeWall;
import com.nonesole.web.platform.sys.rightset.RightSetEnum;
import com.nonesole.web.platform.sys.user.User;
import com.nonesole.web.platform.sys.user.UserEnum;

/**
 * Check whether the visitor could see the resource.
 * @author jack lee
 * @version 1.0
 * @date 2016-10-17
 */
public class AccessFilter implements Filter {

	public static final String COOKIE_SWITCH = "cookie";
	public static final String COOKIE_OPEN = "1";
	public static final String COOKIE_KEY_TIME = "logintime";
	public static final String COOKIE_KEY_USER_NAME = "username";
	public static final String COOKIE_KEY_MD5_CODE = "md5code";
	
	protected Logger logger;
	public static Map<String,ServletRegistration> servletRegistrations;
	public static final String SERVLET_NAME_JSP = "jsp";
	public static final String SERVLET_NAME_DEFAULT = "default";

	public static final String LOGIN_URL="/login.jsp";
	public static final String ACTION_URL="/loginAction";
	public static final String LOGOUT_ACTION_URL="/logoutAction";
	
	@Override
	public void init(FilterConfig config) throws ServletException {
		//logger = logger.getInstance();
		ActionControler.getInstance();
		this.initServletCache(config);
	}
	
	/**
	 * Init servlet cache
	 * @param config
	 */
	@SuppressWarnings("unchecked")
	public void initServletCache(FilterConfig config){
		if ( servletRegistrations != null ) {
			return;
		}
		ServletContext sc = config.getServletContext();
		sc.getServletRegistrations();
		Map<String, ? extends ServletRegistration> map = sc.getServletRegistrations();
		if ( map != null ) {
			Set<?> ens = map.entrySet();
			Entry<String, ServletRegistration> sr = null;
			servletRegistrations = new HashMap<String,ServletRegistration>();
			for ( Object obj : ens ){
				sr = (Entry<String,ServletRegistration>)obj;
				if ( SERVLET_NAME_JSP.equals(sr.getKey()) || 
						SERVLET_NAME_DEFAULT.equals(sr.getKey()) ) {
					continue;
				}
				servletRegistrations.put(sr.getKey(), sr.getValue());
			}
		}
	}
	
	@Override
	public void destroy() {
		
	}

	/**
	 * To login page
	 * @param request
	 * @param response
	 * @throws IOException 
	 */
	public void toLoginPage(HttpServletRequest request, HttpServletResponse response) 
			throws IOException{
		String url = request.getRequestURL().toString();
		String uri = request.getRequestURI();
		String contextPath = request.getContextPath();
		StringBuilder path = new StringBuilder();
		path.append(url.replace(uri, "")).append(contextPath).append("/").append(ParameterEnum.URL_LOGIN.toString());
		response.sendRedirect(path.toString());
	}
	
	/**
	 * To some page
	 * @param request
	 * @param response
	 * @throws IOException 
	 */
	public void toPage(HttpServletRequest request, HttpServletResponse response, String pageName) 
			throws IOException{
		String url = request.getRequestURL().toString();
		String uri = request.getRequestURI();
		String contextPath = request.getContextPath();
		StringBuilder path = new StringBuilder();
		path.append(url.replace(uri, "")).append(contextPath).append("/").append(pageName);
		response.sendRedirect(path.toString());
	}
	
	@Override
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {		
		String encoding = "utf-8";
		request.setCharacterEncoding(encoding); 
		response.setContentType("text/html;charset="+encoding); 
		
		HttpServletRequest httpRequest = (HttpServletRequest) request;
		HttpServletResponse httpResponse = (HttpServletResponse) response;
		
		if ( this.isLogoutAction(httpRequest) ){
			this.clearUserLoginSession(httpRequest);
			LoginCookies lc = new LoginCookies();
			lc.clearCookies(httpResponse);
			this.toLoginPage(httpRequest, httpResponse);
			return;
		}
		
		if ( this.isLoginAction(httpRequest) ) {
			try {
				this.login(httpRequest, httpResponse);
				return;
			} catch (ExecuteSQLException e) {
				//logger.error("There is an error on login action.", e);
				this.toLoginPage(httpRequest, httpResponse);
				return;
			} 
		} else {
			try {
				if ( !this.jspFilter(httpRequest, httpResponse) ) {
					chain.doFilter(request, response);
					return;
				} else if ( !this.actionFilter(httpRequest, httpResponse) ) {
					chain.doFilter(request, response);
					return;
				} else {
					//Other resources
				}
			} catch (ExecuteSQLException e) {
				//logger.error("There is an error on user-check action.", e);
				this.toLoginPage(httpRequest, httpResponse);
				return;
			}
		}
		chain.doFilter(request, response);
	}
	
	/**
	 * Servlet filter
	 * @param httpRequest
	 * @param httpResponse
	 * @return
	 * @throws IOException 
	 * @throws ExecuteSQLException 
	 */
	private boolean actionFilter(HttpServletRequest httpRequest, HttpServletResponse httpResponse) 
			throws IOException, ExecuteSQLException {
		if ( PlatformTools.isServlet(httpRequest) ) {
			User user = this.getUser(httpRequest, httpResponse);
			if ( !this.validateUser(user) ) {
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			}
			if ( ActionControler.getInstance().isUserLocked(user.getId()) ) {
				//logger.warn("[Permission denied]The user[" + user.getLogin_name() + "] had visited url["+ path +"]." , null);
				this.clearUserLoginSession(httpRequest);
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			}
			
			if ( ActionControler.actionParameter2RoleIdRelationMap == null ) {
				this.clearUserLoginSession(httpRequest);
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			}
			
			List<Integer> roles = this.getRoles(httpRequest);
			if ( roles == null ) {
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			} else {
				if ( !this.checkActionParameterForUser(httpRequest, roles) ) {
					this.clearUserLoginSession(httpRequest);
					this.toLoginPage(httpRequest, httpResponse);
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * JSP filter
	 * @param httpRequest
	 * @param httpResponse
	 * @return
	 * @throws IOException
	 * @throws ExecuteSQLException 
	 */
	private boolean jspFilter(HttpServletRequest httpRequest, HttpServletResponse httpResponse) 
			throws IOException, ExecuteSQLException{
		if ( PlatformTools.isJSP(httpRequest) ) {
			User user = this.getUser(httpRequest, httpResponse);
			String path = httpRequest.getServletPath();
			if ( LOGIN_URL.equals(path) || SafeWall.CODE_MAKER.equals(path) ){
				if ( LOGIN_URL.equals(path) ) {
					if ( this.validateUser(user) ) {
						this.toPage(httpRequest, httpResponse, ParameterEnum.URL_BACK_INDEX.toString());
						return false;
					}
				}
				return false;
			}
			
			if ( !this.validateUser(user) ) {
				//logger.warn("[Permission denied]The user's info is wrong." , null);
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			}
			
			if ( ActionControler.getInstance().isUserLocked(user.getId()) ) {
				//logger.warn("[Permission denied]The user[" + user.getLogin_name() + "] had visited url["+ path +"]." , null);
				this.clearUserLoginSession(httpRequest);
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			}
			
			List<Integer> roles = this.getRoles(httpRequest);
			if ( roles == null ) {
				
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			} else {
				
				if ( ActionControler.commonBackPages.get(path) != null ) {
					return false;
				} 

				if ( ActionControler.adminBackPages.get(path) != null ) {
					if ( this.isAdmin(roles) ) {
						return false;
					} else {
						//logger.warn("[Permission denied]The user[" + user.getLogin_name() + "] had visited url["+ path +"]." , null);
						this.clearUserLoginSession(httpRequest);
						this.toLoginPage(httpRequest, httpResponse);
						return false;
					}
				}
				if ( ActionControler.url2RoleIdMap == null ) {
					this.clearUserLoginSession(httpRequest);
					this.toLoginPage(httpRequest, httpResponse);
					return false;
				}
				if ( !this.checkUrlForUser(httpRequest) ) {
					//logger.warn("[Permission denied]The user[" + user.getLogin_name() + "] had visited url["+ path +"]." , null);
					this.clearUserLoginSession(httpRequest);
					this.toLoginPage(httpRequest, httpResponse);
					return false;
				}
			}
		}
		return true;
	}
	
	/**
	 * Check whether or not the user is logout
	 * @param httpRequest
	 * @return
	 */
	private boolean isLogoutAction(HttpServletRequest httpRequest) {
		String path = httpRequest.getServletPath();
		if ( LOGOUT_ACTION_URL.equals(path) ) {
			//logger.debug("AccessFilter:isLogout:"+path, null);
			return true;
		}
		return false;
	}
	
	/**
	 * Check whether or not the user is login
	 * @param httpRequest
	 * @return
	 */
	private boolean isLoginAction(HttpServletRequest httpRequest) {
		String path = httpRequest.getServletPath();
		if ( ACTION_URL.equals(path) ) {
			if ( SafeWall.REDIRECT_SOURCE_TARGET != null ) {
				String url = SafeWall.REDIRECT_SOURCE_TARGET.get(LOGIN_URL);
				if ( !PlatformTools.isNull(url) ) {
					if ( url.equals(this.getRefererPageUrl(httpRequest)) ) {
						return true;
					} else {
						return false;
					}
				} 
			}
			if ( LOGIN_URL.equals(this.getRefererPageUrl(httpRequest)) ) {
				return true;
			}	
		}
		return false;
	}

	/**
	 * Login
	 * @param httpRequest
	 * @throws IOException 
	 * @throws ExecuteSQLException 
	 */
	private void login(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws IOException, ExecuteSQLException{
		if ( !SafeWall.checkCode(httpRequest) ) {
			PlatformTools.setCookieMessage(httpResponse, 
					LocaleToolkit.getLocaleString(httpRequest, "com.nonesole.web.platform.sys.AccessFilter_info_1"));
			this.toLoginPage(httpRequest, httpResponse);
			return;
		}
		
		Map<String,Object> c = new HashMap<String,Object>();
		String key = httpRequest.getParameter(UserEnum.PARAM_LOGIN_KEY.toString());
		
		c.put(UserEnum.PARAM_LOGIN_KEY.toString(), PlatformTools.md5(key));
		
		String actionName = null;
		String input = httpRequest.getParameter(UserEnum.PARAM_LOGIN_NAME.toString());
		
		if ( PlatformTools.isEmail(input) ) {
			actionName = UserEnum.DAO_GET_USER_BY_EMAIL.toString();
			c.put(UserEnum.PARAM_LOGIN_EMAIL.toString(), input);
		} else if ( PlatformTools.validateUserName(input) ) {
			actionName = UserEnum.DAO_GET_USER_BY_USER_NAME.toString();
			c.put(UserEnum.PARAM_LOGIN_NAME.toString(), input);
		} else if( PlatformTools.isCellPhoneNumber(input) ) {
			actionName = UserEnum.DAO_GET_USER_BY_CELL_PHONE_NUMBER.toString();
			c.put(UserEnum.PARAM_LOGIN_CELL_PHONE_NUMBER.toString(), input);
		} else {
			SafeWall.countLoginError(httpRequest);
			PlatformTools.setCookieMessage(httpResponse, LocaleToolkit.getLocaleString(httpRequest, "com.nonesole.web.platform.sys.AccessFilter_info_2"));
			this.toLoginPage(httpRequest, httpResponse);
			return;
		}
		
		boolean validate = this.checkUser(httpRequest, httpResponse, actionName, c , null);
		if ( !validate ) {
			return;
		}
	
		SafeWall.cleanLoginErrorInfo(httpRequest);
				
		httpResponse.sendRedirect(ParameterEnum.URL_BACK_INDEX.toString());
		
	}
	
	/**
	 * Check user
	 * @param httpRequest
	 * @param httpResponse
	 * @param actionName
	 * @param values
	 * @throws IOException 
	 * @throws ExecuteSQLException 
	 */
	private boolean checkUser(HttpServletRequest httpRequest, HttpServletResponse httpResponse, 
			String actionName, Map<String,Object> values, LoginCookies lc) throws IOException, ExecuteSQLException{
		@SuppressWarnings("unchecked")
		IPojoAction<User> acts = (IPojoAction<User>) PersistenceFactory.getInstance().getPojoAction();
		User u = null;
		try {
			u = acts.pojoAction(actionName, values);
		} catch (ExecuteSQLException e) {
			logger.error("[AccessFilter:login]", e);
		}
		if ( u == null || u.getId() < 1 ) {
			SafeWall.countLoginError(httpRequest);
			PlatformTools.setCookieMessage(httpResponse, 
					LocaleToolkit.getLocaleString(httpRequest, "com.nonesole.web.platform.sys.AccessFilter_info_2"));
			this.toLoginPage(httpRequest, httpResponse);
			return false;	
		}
		
		if ( lc != null ) {
			lc.setMd5password(PlatformTools.md5(u.getLogin_key()));
			if ( !lc.validate() ) {
				PlatformTools.setCookieMessage(httpResponse, 
						LocaleToolkit.getLocaleString(httpRequest, "com.nonesole.web.platform.sys.AccessFilter_info_2"));
				this.toLoginPage(httpRequest, httpResponse);
				return false;
			} else {
				lc.setLoginTime(System.currentTimeMillis()).updateCookies(httpResponse);
			}
		}
		
		httpRequest.getSession().setAttribute(UserEnum.ATTR_SESSION_USER.toString(), u);
		
		IListValueAction sAction = PersistenceFactory.getInstance().getListValueAction();
		Map<String, Object> cd = new HashMap<String, Object>();
		cd.put(RightSetEnum.PARAM_CONDITION_ID.toString(), "#uid#");
		cd.put("#uid#", String.valueOf(u.getId()));
		List<Integer> rolesId = sAction.intListAction(RightSetEnum.DAO_FIND_ROLES_BY_USER_ID.toString(), cd);
		if ( rolesId != null ) {
			httpRequest.getSession().setAttribute(UserEnum.ATTR_SESSION_USER_ROLES.toString(), rolesId);
		}

		if (u != null) {
			Object me = httpRequest.getParameter(UserEnum.PARAM_REMEMBER_ME.toString());
			if ( PlatformTools.isNull(me) ) {
				return true;
			} else {
				if ( UserEnum.PARAM_VALUE_REMEMBER_ME.toString().equals(me) ) {
					lc = new LoginCookies();
					lc.setLoginTime(System.currentTimeMillis())
					  .setLoginName(u.getLogin_name())
					  .setMd5password(PlatformTools.md5(u.getLogin_key()))
					  .updateCookies(httpResponse);
				}
			}
		}
		return true;
	}
		
	/**
	 * Get referer
	 * @param httpRequest
	 * @return
	 */
	private String getReferer(HttpServletRequest httpRequest){
		Object referer = null;
		if ( httpRequest.getHeader("Referer") == null || "null".equals(httpRequest.getHeader("Referer")) ) {
			referer = httpRequest.getSession().getAttribute("from_index_referer");
		} else {
			referer = httpRequest.getHeader("Referer");
		}
		referer = referer==null?"":(String)referer;
		return referer.toString();
	}
	
	/**
	 * Get referer
	 * @param httpRequest
	 * @return
	 */
	private String getRefererPageUrl(HttpServletRequest httpRequest){
		String referer = this.getReferer(httpRequest);
		int startIndex = referer.lastIndexOf("/");
		if ( startIndex < 0 ) {
			return "";
		}
		referer = referer.substring(startIndex);
		//logger.debug("getRefererPageUrl:"+referer, null);
		return referer;
	}
	
	/**
	 * Check user
	 * @param httpRequest
	 * @return
	 * @throws ExecuteSQLException 
	 * @throws IOException 
	 */
	private boolean validateUser(User u) throws IOException, ExecuteSQLException {
		if ( u != null) {
			return u.validate(User.CHECK);
		} 
		return false;
	}
	
	/**
	 * Get user from session
	 * @param httpRequest
	 * @return
	 * @throws IOException 
	 * @throws ExecuteSQLException 
	 */
	public User getUser(HttpServletRequest request, HttpServletResponse response) 
			throws IOException, ExecuteSQLException {
		Object obj = request.getSession().getAttribute(UserEnum.ATTR_SESSION_USER.toString());
		if ( obj != null) {
			User u = (User)obj;
			return u;
		} else {
			LoginCookies lc = new LoginCookies();
			lc.readCookies(request);
			if ( lc.validateParameter() ) {
				Map<String,Object> c = new HashMap<String,Object>();
				String actionName = UserEnum.DAO_GET_USER_BY_COOKIES.toString();
				c.put(UserEnum.PARAM_LOGIN_NAME.toString(), lc.getLoginName());
				this.checkUser(request, response, actionName, c , lc);
				
				SafeWall.cleanLoginErrorInfo(request);
				obj = request.getSession().getAttribute(UserEnum.ATTR_SESSION_USER.toString());
				if ( obj != null) {
					User u = (User)obj;
					return u;
				}
			} 
		}
		return null;
	}
	
	/**
	 * Get roles of user
	 * @param httpRequest
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private List<Integer> getRoles(HttpServletRequest httpRequest){
		Object l = httpRequest.getSession().getAttribute(UserEnum.ATTR_SESSION_USER_ROLES.toString());
		if ( l == null ) {
			return null;
		} else {
			return (List<Integer>)l;
		}
	}
	
	/**
	 * Clear session
	 * @param httpRequest
	 */
	private void clearUserLoginSession(HttpServletRequest httpRequest){
		httpRequest.getSession().setAttribute(UserEnum.ATTR_SESSION_USER.toString(), null);
		httpRequest.getSession().setAttribute(UserEnum.ATTR_SESSION_USER_ROLES.toString(), null);
	}
	
	/**
	 * Check whether or not the user is admin
	 * @param httpRequest
	 * @return
	 */
	public final static boolean isAdmin(HttpServletRequest request) {
		//logger.getInstance().debug("isAdmin:"+request.getSession(), null);
		Object l = request.getSession().getAttribute(UserEnum.ATTR_SESSION_USER_ROLES.toString());
		if ( l == null ) {
			return false;
		} 
		@SuppressWarnings("unchecked")
		List<Integer> roles = (List<Integer>)l;
		if ( roles != null) {
			for ( int role : roles ) {
				if ( role == 1 ) {
					//logger.getInstance().debug("isAdmin:true", null);
					return true;
				}
			}
		}
		
		return false;
	}
	
	/**
	 * Check whether or not the user is admin
	 * @param httpRequest
	 * @return
	 */
	private boolean isAdmin(List<Integer> roles) {
		if ( roles != null) {
			for ( int role : roles ) {
				if ( role == 1 ) {
					//role id 为1 说明是系统创建之初就建立的账户管理员角色
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Check whether or not the user can access the url
	 * @param httpRequest
	 * @return
	 */
	private boolean checkUrlForUser(HttpServletRequest httpRequest) {
		Object l = httpRequest.getSession().getAttribute(UserEnum.ATTR_SESSION_USER_ROLES.toString());
		if ( l == null ) {
			return false;
		} 
		String path = httpRequest.getServletPath();
		Integer roleId = 0;
		@SuppressWarnings("unchecked")
		List<Integer> roles = (List<Integer>)l;
		if ( roles != null) {
			for ( int role : roles ) {
				if ( role > 1 ) {
					roleId = ActionControler.url2RoleIdMap.get(path+"_"+role);
					if ( roleId != null && roleId > 0 ) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
	/**
	 * Check whether or not the user can access action
	 * @param httpRequest
	 * @return
	 */
	private boolean checkActionParameterForUser(HttpServletRequest httpRequest, List<Integer> userRoles) {
		String actionName = httpRequest.getParameter(ParameterEnum.ACTION.toString());
		Integer roleId = ActionControler.actionParameter2RoleIdRelationMap.get(actionName);
		if ( roleId == null ) {
			return false;
		} else {
			for ( int rId : userRoles ){
				if ( rId == roleId ) {
					return true;
				}
			}
		}
		return true;
	}
}
