/**
 * Roya is a micro-core web-application platform which is designed with MVC. 
 * It has easy-use functions for authority management which is based on 
 * platform. This platform is build with Persistence2 framework and could 
 * meet developer’s need of software quick-development and publish.
 * 
 * The MIT License (MIT)
 * 
 * Copyright (c) 2017, nonesole.com
 * 
 * Permission is hereby granted, free of charge, to any person obtaining 
 * a copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
 * THE SOFTWARE.
 */
package com.nonesole.web.platform.sys.menu;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import com.nonesole.log.Logger;
import com.nonesole.web.platform.TreeNode;
import com.nonesole.web.platform.i18n.LocaleToolkit;
import com.nonesole.web.platform.sys.CoreException;

/**
 * MenuTree
 * @author jack lee
 * @version 1.0
 * @date 2016-10-12
 */
public class MenuTree {

	private List<Menu> menus = null;
	private List<TreeNode<Menu>> topNodes = null;
	private Map<Integer,Integer> pidMap = null;
	private Map<Integer,TreeNode<Menu>> nodeMap = null;
	private String topMenuStr = null;
	private String leftMenuStr = null;
	
	/**
	 * Constructor
	 * @param list
	 * @throws CoreException
	 */
	public MenuTree(List<Menu> list) throws CoreException {
		this.menus = list;
		if ( this.menus == null ) {
			throw new CoreException("There is no nodes for tree.");
		}
		this.initTree();
	}
	
	
	
	/**
	 * Constructor
	 * @param list
	 * @param selectedId
	 * @throws CoreException
	 */
	public MenuTree(List<Menu> list, List<Integer> selectedId) throws CoreException {
		this(list);
		this.menus = this.getSelectedMenu(selectedId);
		this.initTree();
		if ( this.topNodes != null ) {
			StringBuilder sb = new StringBuilder();
			Collections.sort(this.topNodes);
			for ( TreeNode<Menu> n : this.topNodes ) {
				sb.append(n.getTopMenuHtml());
			}
			topMenuStr = sb.toString();
			
			StringBuilder sb1 = new StringBuilder();
			Set<Integer> set = new HashSet<Integer>();
			if ( this.pidMap != null && this.pidMap.values() != null ) {
				set.addAll(this.pidMap.values());
				List<Integer> sl = new ArrayList<Integer>(set);
				Collections.sort(sl);
				Logger.getInstance().debug("sort:" + sl, null);
				for ( Integer id : sl ) {
					Logger.getInstance().debug("MenuTree:init:"+this.nodeMap.get(id), null);
					if ( this.nodeMap.get(id) != null ) {
						sb1.append(this.nodeMap.get(id).getLeftMenuHtml()).append("\r\n");
					}
				}
				leftMenuStr = sb1.toString();
			}		
		}
		
	}
	
	/**
	 * init tree
	 */
	public void initTree(){
		this.topNodes = new ArrayList<TreeNode<Menu>>();
		this.pidMap = new HashMap<Integer,Integer>();
		this.nodeMap = new HashMap<Integer,TreeNode<Menu>>();
		TreeNode<Menu> treeNode = null;
		for ( Menu m : this.menus ) {
			treeNode = new TreeNode<Menu>();
			treeNode.setObj(m);
			treeNode.setId(m.getId());
			treeNode.setNodeName(m.getName());
			treeNode.setParentId(m.getPid());
			treeNode.setUrl(m.getUrl());
			treeNode.setIco(m.getIco());
			this.pidMap.put(m.getId(), m.getPid());
			this.nodeMap.put(m.getId(), treeNode);
			if ( m.getPid() == 0 ) {
				this.topNodes.add(treeNode);
			}
		}
		for ( Menu m : this.menus ) {
			treeNode = this.nodeMap.get(m.getPid());
			if ( treeNode != null ) {
				this.nodeMap.get(m.getId()).setParentNode(treeNode);
				treeNode.addChildNode(this.nodeMap.get(m.getId()));
			}
		}
	}
	
	/**
	 * Get objects from tree
	 * @return
	 * @throws CoreException
	 */
	public List<Menu> getObjects(Locale locale) throws CoreException{
		if ( topNodes == null ) {
			throw new CoreException("There is not top node.");
		}
		Collections.sort(topNodes);
		List<Menu> l = new ArrayList<Menu>();
		List<TreeNode<Menu>> nodes = new ArrayList<TreeNode<Menu>>();
		for ( TreeNode<Menu> node : topNodes ) {
			nodes.add(node);
			nodes.addAll(node.getAllChildren());
		}
		for ( TreeNode<Menu> tn : nodes ) {
			Logger.getInstance().debug("Tree Node : " + tn.getObj(), null);
			tn.getObj().setLevel(tn.getLevel());
			tn.getObj().setLeaf(tn.isLeaf()?1:0);//0false，1true
			if ( tn.getObj().getLeaf() == 0 ) {
				tn.getObj().setUrl(LocaleToolkit.getLocaleString(locale, "com.nonesole.web.platform.sys.menu.MenuTree_info_1"));
			}
			l.add(tn.getObj());
		}
		return l;
	}
	
	/**
	 * Get json of tree
	 * @return
	 * @throws CoreException
	 */
	public String getJSON(Locale locale) throws CoreException{
		List<Menu> l = this.getObjects(locale);
		if ( l != null && !l.isEmpty() ) {
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			int lastIndex = l.size()-1;
			Menu m = null;
			for ( int i = 0 ; i < l.size() ; i++ ) {
				m = l.get(i);
				sb.append("{")
				  .append("id:").append(m.getId()).append(",")
				  .append("pId:").append(m.getPid()).append(",")
				  .append("name:\"").append(m.getName()).append("\"");
				if ( m.getLeaf() != 1 ) {
					sb.append(",open:true");
				}
				sb.append("}");
				if ( i < lastIndex ) {
					sb.append(",");
				}
			}
			sb.append("]");
			return sb.toString();
		}
		return "[]";
	}
	
	/**
	 * Get node-selected json of tree
	 * @return
	 * @throws CoreException
	 */
	public String getSelectedJSON(Locale locale, List<Integer> selectedObject) throws CoreException{
		List<Menu> l = this.getObjects(locale);
		if ( l != null && !l.isEmpty() ) {
			StringBuilder sb = new StringBuilder();
			sb.append("[");
			int lastIndex = l.size()-1;
			Menu m = null;
			for ( int i = 0 ; i < l.size() ; i++ ) {
				m = l.get(i);
				sb.append("{")
				  .append("id:").append(m.getId()).append(",")
				  .append("pId:").append(m.getPid()).append(",")
				  .append("name:\"").append(m.getName()).append("\"");
				if ( m.getLeaf() != 1 ) {
					sb.append(",open:true");
				}
				if ( selectedObject != null && selectedObject.contains(m.getId()) ) {
					sb.append(",checked:true");
				}
				sb.append("}");
				if ( i < lastIndex ) {
					sb.append(",");
				}
			}
			sb.append("]");
			return sb.toString();
		}
		return "[]";
	}
	
	/**
	 * Get selected leaves of menu
	 * @return
	 * @throws CoreException
	 */
	public List<Menu> getSelectedLeafMenu(List<Integer> selectedId) throws CoreException{
		List<Menu> menus = new ArrayList<Menu>();
		if ( selectedId != null ) {
			TreeNode<Menu> node = null;
			for ( int id : selectedId ) {
				node = this.nodeMap.get(id);
				if ( node != null ){
					node.setSelected();
					node.updateParent();
				}
			}
		}
		if ( this.topNodes != null ) {
			Set<Menu> set = new HashSet<Menu>();
			for ( TreeNode<Menu> node : topNodes ) {
				node.updateChildren();
				set.addAll(node.getSelectedLeafObjects());
			}
			menus.addAll(set);
		}
		Logger.getInstance().debug(">>> getSelectedMenu : " + menus, null);
		return menus;
	}
	
	/**
	 * Get selected-node menu
	 * @return
	 * @throws CoreException
	 */
	public List<Menu> getSelectedMenu(List<Integer> selectedId) throws CoreException{
		List<Menu> menus = new ArrayList<Menu>();
		if ( selectedId != null ) {
			TreeNode<Menu> node = null;
			for ( int id : selectedId ) {
				node = this.nodeMap.get(id);
				if ( node != null ){
					node.setSelected();
				}
			}
		}
		if ( this.topNodes != null ) {
			Set<Menu> set = new HashSet<Menu>();
			for ( TreeNode<Menu> node : topNodes ) {
				node.checkSelectedValidate();
				set.addAll(node.getSelectedObjects());
			}
			menus.addAll(set);
		}
		//Logger.getInstance().info(">>> getSelectedMenu : " + menus, null);
		return menus;
	}
	
	/**
	 * Get id list of selected nodes
	 * @return
	 * @throws CoreException
	 */
	public List<Integer> getSelectedNodesId(List<Integer> selectedId) throws CoreException{
		List<Integer> l = new ArrayList<Integer>();
		List<Menu> menus = this.getSelectedMenu(selectedId);
		if ( menus != null ) {
			for ( Menu m : menus ) {
				l.add(m.getId());
			}
		}
		return l;
	}
	
	/**
	 * Get id list of selected leaf nodes
	 * @return
	 * @throws CoreException
	 */
	public List<Integer> getSelectedLeafNodesId(List<Integer> selectedId) throws CoreException{
		List<Integer> l = new ArrayList<Integer>();
		List<Menu> menus = this.getSelectedLeafMenu(selectedId);
		if ( menus != null ) {
			for ( Menu m : menus ) {
				l.add(m.getId());
			}
		}
		return l;
	}
	
	/**
	 * Get html of top menu
	 * @return
	 */
	public String getTopMenu(){
		Logger.getInstance().debug("topMenu:"+this.topMenuStr, null);
		return this.topMenuStr;
	}
	
	/**
	 * Get html of left menu
	 * @return
	 */
	public String getLeftMenu(){
		Logger.getInstance().debug("leftMenuStr:"+this.leftMenuStr, null);
		return this.leftMenuStr;
	}
	
	/**
	 * Get first node id
	 * @return
	 */
	public int getFirstTopNodeId(){
		if ( this.topNodes != null && this.topNodes.get(0) != null ) {
			return this.topNodes.get(0).getId();
		}
		return 0;
	}
}
